#include <omp.h>
#include <iostream>
#include <iomanip>
#include <cmath>
using namespace std;

// g++ algo_simple_3.cpp -o algo_simple_3 -fopenmp

typedef double DataType;
#define Nthreads 2

#define N        10000000

/* Some random number constants from numerical recipies */
#define SEED       2531
#define RAND_MULT  1366
#define RAND_ADD   150889
#define RAND_MOD   714025


int randy = SEED;

/* function to fill an array with random numbers */
void fill_rand(int length, DataType *a)
{
   int i; 
   for (i=0;i<length;i++) 
	{
     randy = (RAND_MULT * randy + RAND_ADD) % RAND_MOD;
     a[i] = ((DataType) randy)/((DataType) RAND_MOD);
	}   
}


int main ()  
{
   int nthreads = Nthreads;
   double runtime;
   int length = N;
   DataType *tab = new DataType[length];
   fill_rand(length, tab);
   
   runtime = omp_get_wtime();
   double sum_1 = 0;
   for( int i=0; i<length-1; i++)
	   sum_1 += sqrt( (tab[i] - tab[i+1]) * (tab[i] - tab[i+1]) );
   runtime = omp_get_wtime() - runtime;
   cout << "With only one thread, sum is : " << sum_1 << " in " << runtime << "seconds" << endl;
   
   
   double sum_th = 0;
   
   omp_set_num_threads(nthreads);
   runtime = omp_get_wtime();
   #pragma omp parallel for reduction(+:sum_th)
	for( int i=0; i<length-1; i++)
	   sum_th += sqrt( (tab[i] - tab[i+1]) * (tab[i] - tab[i+1]) ); 
    runtime = omp_get_wtime() - runtime;
   cout << "With " << nthreads << "  threads, the sum is : " << sum_th << " in " << runtime << "seconds" << endl;
  
  cout << "Remark : the difference is " << setprecision(15) << sum_th - sum_1 << endl;
return 0;
}

 

