/*

This program will numerically compute the integral of

                  4/(1+x*x) 
				  
from 0 to 1.  The value of this integral is pi -- which 
is great since it gives us an easy way to check the answer.

The program was parallelized using OpenMP by adding just
four lines 

(1) A line to include omp.h -- the include file that 
contains OpenMP's function prototypes and constants.

(2) A pragma that tells OpenMP to create a team of threads

(3) A pragma to cause one of the threads to print the
number of threads being used by the program.

(4) A pragma to split up loop iterations among the team
of threads.  This pragma includes 2 clauses to (1) create a 
private variable and (2) to cause the threads to compute their
sums locally and then combine their local sums into a 
single global value.

History: Written by Tim Mattson, 11/99.
   few changes (C++) by T. Grenier, November 2015
*/
#include <iostream>
#include <iomanip>
#include <omp.h>

using namespace std;

#define Max_Thread 16
static long num_steps = 200000000;

// g++ pi_loop.cpp -o pi_loop -fopenmp

double step;
int main ()
{
	  int i;
	  double x, pi, sum = 0.0;
	  double start_time, run_time;

	  step = 1.0/(double) num_steps;
	 for (i=1;i<=Max_Thread;i++)
		{
        sum = 0.0;
        omp_set_num_threads(i);
		start_time = omp_get_wtime();
		#pragma omp parallel  
			{
			#pragma omp single
			cout << " num_threads = " << omp_get_num_threads() << endl;

			#pragma omp for reduction(+:sum) private(i)
			for ( i=1; i<= num_steps; i++)
				{
				x = (i-0.5)*step;
				sum = sum + 4.0/(1.0+x*x);
				}
			}
		pi = step * sum;
		run_time = omp_get_wtime() - start_time;
		cout << "  pi is " << setprecision(15) << pi << " in " << run_time << " seconds and " << i << "threads " << endl;
		}
	return 0;
}	  





