/*
**  PROGRAM: A simple SPMD producer/consumer program
**
**  PURPOSE: this is just a stupid little program to play around
**  with different ways data is shared between threads.
**
**  HISTORY: Written by Tim Mattson, April 2007.
**        few changes (C++/typedef/reduction) by T. Grenier, November 2015
*/
#include <omp.h>
#include <cstdlib>
#include <iostream>
using namespace std;

#define N        10000000
#define Nthreads 2

/* Some random number constants from numerical recipies */
#define SEED       2531
#define RAND_MULT  1366
#define RAND_ADD   150889
#define RAND_MOD   714025

typedef double DataType;

int randy = SEED;

/* function to fill an array with random numbers */
void fill_rand(int length, DataType *a)
{
   int i; 
   for (i=0;i<length;i++) {
     randy = (RAND_MULT * randy + RAND_ADD) % RAND_MOD;
     *(a+i) = ((DataType) randy)/((DataType) RAND_MOD);
   }   
}

/* function to sum the elements of an array */
DataType Sum_array(int length, DataType *a)
{
   int i;  DataType sum = 0.0;
   #pragma omp parrallel for  reduction(+:sum)
   for (i=0;i<length;i++)  sum += *(a+i); //  sum += a[i];
   return sum; 
}
  
int main()
{
  DataType *A, sum, 
  double runtime;
  int numthreads, flag = 0;

  omp_set_num_threads(Nthreads);

  A = new DataType[N];

  #pragma omp parallel
  {
  #pragma omp master
  {
  numthreads = omp_get_num_threads();
  if(numthreads != Nthreads)
        {
           cout << "error: incorect number of threads. Compile with -fopenmp ?" << numthreads << endl;
           exit( -1);
        }
  }
  }
  runtime = omp_get_wtime();
  
  fill_rand(N, A);
 
  sum = Sum_array(N, A);
        
  runtime = omp_get_wtime() - runtime;
   

   cout << " with " << numthreads<< " threads and " << runtime << " seconds, The sum is " << sum << endl;
   
   
   delete[] A;
   
   return 0;
}
 
